package xyz.lychee.lagfixer.libs.adventure.pointer;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Supplier;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class PointersImpl implements Pointers {
   static final Pointers EMPTY = new Pointers() {
      public <T> @NotNull Optional<T> get(final @NotNull Pointer<T> pointer) {
         return Optional.empty();
      }

      public <T> boolean supports(final @NotNull Pointer<T> pointer) {
         return false;
      }

      public Pointers.@NotNull Builder toBuilder() {
         return new BuilderImpl();
      }

      public String toString() {
         return "EmptyPointers";
      }
   };
   private final Map<Pointer<?>, Supplier<?>> pointers;

   PointersImpl(@NotNull final @NotNull BuilderImpl builder) {
      this.pointers = new HashMap(builder.pointers);
   }

   public <T> @NotNull Optional<T> get(final @NotNull Pointer<T> pointer) {
      Objects.requireNonNull(pointer, "pointer");
      Supplier<?> supplier = (Supplier)this.pointers.get(pointer);
      return supplier == null ? Optional.empty() : Optional.ofNullable(supplier.get());
   }

   public <T> boolean supports(final @NotNull Pointer<T> pointer) {
      Objects.requireNonNull(pointer, "pointer");
      return this.pointers.containsKey(pointer);
   }

   public Pointers.@NotNull Builder toBuilder() {
      return new BuilderImpl(this);
   }

   static final class BuilderImpl implements Pointers.Builder {
      private final Map<Pointer<?>, Supplier<?>> pointers;

      BuilderImpl() {
         this.pointers = new HashMap();
      }

      BuilderImpl(final @NotNull PointersImpl pointers) {
         this.pointers = new HashMap(pointers.pointers);
      }

      @NotNull
      public <T> Pointers.@NotNull Builder withDynamic(final @NotNull Pointer<T> pointer, final @NotNull Supplier<@Nullable T> value) {
         this.pointers.put((Pointer)Objects.requireNonNull(pointer, "pointer"), (Supplier)Objects.requireNonNull(value, "value"));
         return this;
      }

      public @NotNull Pointers build() {
         return new PointersImpl(this);
      }
   }
}
